from typing import Literal
from pydantic import BaseModel, computed_field
from xync_schema.pydantic import BaseAd


# Модели для вложенных структур
class UserStatistics(BaseModel):
    userId: int
    totalOrdersCount: int
    successRate: str
    successPercent: int


class User(BaseModel):
    userId: int
    nickname: str
    avatarCode: str
    statistics: UserStatistics
    isVerified: bool
    lastOnlineMinutesAgo: int | None = None
    onlineStatus: str


class AvailableVolume(BaseModel):
    currencyCode: str  # coin
    amount: str  # of asset


class Price(BaseModel):
    type: Literal["FLOATING", "FIXED"]
    baseCurrencyCode: str
    quoteCurrencyCode: str
    value: str
    estimated: str | None = None


class OrderLimitsIn(BaseModel):
    min: str  # In


class OrderLimits(OrderLimitsIn):
    currencyCode: str
    max: str
    approximate: bool


Status = Literal["ACTIVE", "INACTIVE", "ACTIVATING", "DEACTIVATING"]


class ChangeLogItem(BaseModel):
    status: Status
    createDateTime: str
    initiatorUserId: int | None = None


class ChangeLog(BaseModel):
    items: list[ChangeLogItem]


class TakerFilter(BaseModel):
    accountAge: str
    completedOrders: str
    userRating: str


class Fee(BaseModel):
    rate: str
    availableVolume: AvailableVolume  # of asset


class KeyVal(BaseModel):
    name: Literal["PAYMENT_DETAILS_NUMBER", "PHONE"] = "PAYMENT_DETAILS_NUMBER"
    value: str


class BanksIn(BaseModel):
    name: Literal["BANKS"] = "BANKS"
    value: list[str]


class Bank(BaseModel):
    code: str
    nameRu: str
    nameEn: str


class Banks(BanksIn):
    value: list[Bank]


class Attrs(BaseModel):
    version: Literal["V1"] = "V1"
    values: list[KeyVal]


class AttrsV2In(BaseModel):
    version: Literal["V2"]
    values: list[KeyVal | BanksIn]


class AttrsV2(AttrsV2In):
    values: list[KeyVal | Banks]


class PmEpyd(BaseModel):
    code: str
    name: str
    originNameLocale: str
    nameEng: str


class PmEpydRoot(PmEpyd):
    banks: list[PmEpyd] | None = None


class FiatEpydIn(BaseModel):
    paymentMethodCode: str
    currencyCode: str
    name: str
    attributes: Attrs | AttrsV2In


class FiatEpyd(BaseModel):
    id: int
    userId: int
    paymentMethod: PmEpydRoot
    currency: str
    name: str = ""
    attributes: Attrs | AttrsV2


class InitVolume(BaseModel):
    currencyCode: str  # coin
    amount: str  # of asset


# Основные модели
class _BaseInOutAd(BaseAd):
    type: Literal["PURCHASE", "SALE"]
    price: Price


class _BaseAd(_BaseInOutAd):
    number: str
    orderAmountLimits: OrderLimits
    orderVolumeLimits: OrderLimits
    availableVolume: AvailableVolume

    @computed_field
    @property
    def is_sell(self) -> bool:
        return self.type == "SALE"


class _PmsTrait:
    paymentMethods: list[PmEpydRoot]


class _StatusTrait:
    status: str  # maker | One


class _UserTrait:
    takerFilter: TakerFilter  # taker | One
    user: User  # taker | One


class _OneTrait(_StatusTrait, _UserTrait):
    comment: str
    changeLog: ChangeLog
    createDateTime: str
    paymentConfirmTimeout: Literal["PT3M", "PT15M", "PT30M"]


class _TakerOne(_BaseAd, _OneTrait):
    orderConfirmationTimeout: Literal["PT3M", "PT15M"]
    orderAcceptTimeout: Literal["PT10M"]


class AdTakerSalePurchase(_BaseAd, _PmsTrait, _UserTrait):
    availableVolume: str


class OneAdTakerMakerSale(_TakerOne):
    paymentDetails: list[FiatEpyd]
    fee: Fee


class OneAdTakerPurchase(_TakerOne, _PmsTrait): ...


class AdMakerPurchase(_BaseAd, _PmsTrait, _StatusTrait): ...


class OneAdMakerPurchase(_BaseAd, _OneTrait, _PmsTrait): ...


class AdMakerSale(_BaseAd, _StatusTrait):
    paymentDetails: list[FiatEpyd]


# # #


# In
class _BaseIn(_BaseInOutAd):
    orderRoundingRequired: bool
    orderAmountLimits: OrderLimitsIn
    paymentConfirmTimeout: str
    comment: str
    takerFilter: TakerFilter


class _AdNew(_BaseIn):
    initVolume: AvailableVolume


class _AdUpd(_BaseIn):
    offerId: int
    volume: str  # int


class _SaleInTrait:
    paymentDetailsIds: list[int]


class _PurchaseInTrait:
    paymentMethodCodes: list[str]


class AdMakerNewSale(_AdNew, _SaleInTrait): ...


class AdMakerUpdSale(_AdUpd, _SaleInTrait): ...


class AdMakerNewPurchase(_AdNew, _PurchaseInTrait): ...


class AdMakerUpdPurchase(_AdUpd, _PurchaseInTrait): ...


# class _BaseInOutAd(BaseAd):
#     type: Literal["PURCHASE", "SALE"]
#     price: Price
#     orderAmountLimits: OrderLimits  # cur/fiat
#     paymentConfirmTimeout: Literal["PT3M", "PT15M", "PT30M"] | None = None  # sale?
#
#     @computed_field
#     @property
#     def is_sell(self) -> bool:
#         return self.type == "SALE"
#
#
# class MyAdEpydIn(_BaseInOutAd):
#     comment: str = ""
#     initVolume: InitVolume  # of asset
#     orderRoundingRequired: bool
#     paymentMethodCodes: list[str] | None = None  # purchase
#     paymentDetailsIds: list[int] | None = None  # sale
#
#     @classmethod
#     @model_validator(mode="before")
#     def check_at_least_one_field(cls, values):
#         if values.get("paymentMethodCodes") or values.get("paymentDetailsIds"):
#             return values
#         raise ValueError("paymentMethodCodes or paymentDetailsIds is required")
#
#
# class _BaseAd(_BaseInOutAd):
#     availableVolume: str  # of asset
#     number: str
#     orderVolumeLimits: OrderLimits  # coin/asset
#     paymentDetails: list[FiatEpyd] | None = None  # SALE
#     paymentMethods: list[PmEpydRoot] | None = None  # PURCHASE
#
#
# class AdEpyd(_BaseAd):
#     takerFilter: TakerFilter
#     user: User
#     orderConfirmationTimeout: Literal["PT3M", "PT15M"] | None = None  # purchase?
#     orderAcceptTimeout: Literal["PT10M"] | None = None  # TAKER
#
#     @classmethod
#     @model_validator(mode="before")
#     def check_at_least_one_field(cls, values):
#         if values.get("paymentMethods") or values.get("paymentDetails"):
#             return values
#         raise ValueError("paymentMethods or paymentDetails is required")
#
#
# class MyAdEpyd(_BaseAd):
#     availableVolume: AvailableVolume  # of asset
#     status: Status
#
#
# class _AllAdFullTrait:
#     createDateTime: str
#     changeLog: ChangeLog
#     takerFilter: TakerFilter
#     fee: Fee | None = None
#     comment: str
#
#
# class MyAdFullEpyd(MyAdEpyd, _AllAdFullTrait): ...
#
#
# class AdFullEpyd(AdEpyd, _AllAdFullTrait):
#     availableVolume: AvailableVolume  # of asset
#     status: Status
